import os
from termcolor import colored

# hehe

class printf:
    """
    `@!{text}$&` => Bold text \n
    `@?{text}$&` => Italic text \n
    `@~{text}$&` => Dim text \n
    `@_{text}$&` => Underlined text \n


    """

    def __filter(text: str) -> str:
        res = (
            text.replace("@!", "\033[1m")
            .replace("@?", "\033[3m")
            .replace("@~", "\033[2m")
            .replace("@_", "\033[4m")
            .replace("$&", "\033[0m")
        )
        return res

    def __rm_filter(text: str) -> str:
        res = (
            text.replace("@!", "")
            .replace("@?", "")
            .replace("@~", "")
            .replace("@_", "")
            .replace("$&", "")
        )
        return res

    def __init__(self, *args, **kwargs) -> None:
        args_filtered = []
        for arg in args:
            args_filtered.append(
                printf.__filter(f"{arg}")
            )

        print(*args_filtered, **kwargs)
        return None

    def full_line(*args, **kwargs) -> None:
        terminal_width: int = os.get_terminal_size().columns
        joiner = " "
        if kwargs.get("sep"):
            joiner = kwargs.get("sep")

        content = joiner.join(args)

        printf(f'{printf.__filter(content)}{" "*(terminal_width - len(printf.__rm_filter(content)))}', **kwargs)

    def full() -> None:
        for i in range(os.get_terminal_size().lines - 1):
            printf.full_line(end="\n")

    def endl(times: int = 1):
        print("\n" * times, end="")

    def title(content: str, line_char: chr = "─") -> None:
        width: int = os.get_terminal_size().columns
        mid_text: str = f" {content} "
        side_width: int = int((width - len(printf.__rm_filter(mid_text))) / 2)

        if len(line_char) != 1:
            raise ValueError("line char is not a char")

        sep_text = line_char * side_width
        printf.endl(5)
        printf(f"{sep_text}{mid_text}{sep_text}")
        printf.endl()

    def clear_screen() -> None:
        # Clear the terminal screen using ANSI escape code
        os.system("cls" if os.name == "nt" else "clear")



import pip

def require(package: str) -> object:
    """Implementation of Node.js's `require`. \n
    This function imports the modules at runtime, but if it cannot find them, it uses pip to install them

    Args:
        package (str): the name of the package

    Returns:
        object: the module
    """
    try:
        return __import__(package)
    except ImportError:
        pip.main(['install', package])
        return __import__(package)
    finally:
        print(f"Pakcage \"{package}\" couldn't be installed, as a resoult require couldn't import it :(")


import zenyx
import databank as db


def test_one():
    zenyx.pyon.debug.init(__file__)
    print(zenyx.pyon.deep_serialize(db.demo_obj))
    assert zenyx.pyon.deep_serialize(db.demo_obj) == {
        "param": [{"param": {"test": [10, 10]}, "PYON_TYPE": "test"}, "asd"],
        "PYON_TYPE": "test",
    }, "Deep Serialize"



"""
# zenyx
version 1.0.1\n

## pyon (python obejct notation)
Enables convertion from objects into JSON and back. 
Just use the common JSON functions such as:
 - `dump`: save an `object`, a `dict` or a `list` into a `.json` file
 - `load`: load an `object`, a `dict` or a `list` from a `.json` file
 - `dumps`: convert an `object`, a `dict` or a `list` into a JSON object (string)
 - `loads`: convert a JSON object (string) into an `object`, a `dict` or a `list`

## object streaming
Watcher: reload object array on json file change. 
Enables the continous loading of a json file.\n
Implemented in: `object_stream`

## require
Runtime import and/or install of modules
Implemented in: `require`

## printf
A better printing solution which naturally contains bold (`@!`), italic (`@?`), dark (`@~`) and underlined (`@_`) text \n
Includes the reset (`$&`) symbol \n
### Other Capabilities:
- `printf.full_line(<args>, <kwargs>)` -> Prints a whole line with `content + " "*remaining_line_length`
- `printf.endl(<amount = 0>)` -> An easy way to print multiple `\n`s
- `printf.title(<content>, <line_char>)` -> Prints the title to the middle of the console using the line_char as the spacing chartacter
"""

from zenyx import pyon, require, streams
from zenyx.console import printf
from zenyx.args import Arguments

